@@ -1,5 +1,5 @@
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2001 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -39,7 +39,7 @@
 int gethostname(const char *, size_t);
 #endif
 
-static const char rcsid[]="$Id: makemime.c,v 1.4 2000/12/31 01:50:22 mrsam Exp $";
+static const char rcsid[]="$Id: makemime.c,v 1.5 2001/05/19 04:01:34 mrsam Exp $";
 
 struct arg_list {
 	struct arg_list *next;
@@ -186,7 +186,8 @@ static void usage()
 {
 	fprintf(stderr,
 "Usage:\n"
-"  makemime -c type [-o file] [-e encoding] [-a \"Header: Contents\"] file\n"
+"  makemime -c type [-o file] [-e encoding] [-C charset] [-N name] \\\n"
+"                   [-a \"Header: Contents\"] file\n"
 "           -m [ type ] [-o file] [-e encoding] [-a \"Header: Contents\"] file\n"
 "           -j [-o file] file1 file2\n"
 "           @file\n"
@@ -198,8 +199,10 @@ static void usage()
 "\n"
 "Options:\n"
 "\n"
-"  -c type         - create a new mime section from \"file\" with this\n"
+"  -c type         - create a new MIME section from \"file\" with this\n"
 "                    Content-Type: (default is application/octet-stream).\n"
+"  -C charset      - MIME charset of a new text/plain section.\n"
+"  -N name         - MIME content name of the new mime section.\n"
 "  -m [ type ]     - create a multipart mime section from \"file\" of this\n"
 "                    Content-Type: (default is multipart/mixed).\n"
 "  -e encoding     - use the given encoding (7bit, 8bit, quoted-printable,\n"
@@ -248,6 +251,8 @@ struct mimestruct {
 		/* The new mime type, and encoding (-e) */
 	const char *mimetype;
 	const char *mimeencoding;
+	const char *textplaincharset;
+	const char *contentname;
 
 		/* A list of -a headers */
 	struct arg_list *a_first, *a_last;
@@ -395,6 +400,48 @@ char **argv= *argvp;
 			continue;
 		}
 
+		if (strncmp(argv[0], "-C", 2) == 0)
+		{
+			char *f=argv[0]+2;
+
+			++argv;
+			--argc;
+
+
+			if (*f == 0)
+			{
+				if (!argc)	usage();
+				f=argv[0];
+				++argv;
+				--argc;
+			}
+			while (isspace((int)(unsigned char)*f))
+				++f;
+			m->textplaincharset=f;
+			continue;
+		}
+
+		if (strncmp(argv[0], "-N", 2) == 0)
+		{
+			char *f=argv[0]+2;
+
+			++argv;
+			--argc;
+
+
+			if (*f == 0)
+			{
+				if (!argc)	usage();
+				f=argv[0];
+				++argv;
+				--argc;
+			}
+			while (isspace((int)(unsigned char)*f))
+				++f;
+			m->contentname=f;
+			continue;
+		}
+
 		if (strncmp(argv[0], "-e", 2) == 0)
 		{
 		char *f=argv[0]+2, *q;
@@ -636,16 +683,42 @@ struct	arg_list *a;
 		if (strcmp(m->mimetype, "auto") == 0)
 			m->mimetype=
 				strcmp(m->mimeencoding, "base64") == 0
-				? "application/octet-stream":"text/plain";
+				? (m->textplaincharset=0,
+				   "application/octet-stream"):"text/plain";
 	}
 
-
 	for (a=m->a_first; a; a=a->next)
 		fprintf(m->outputfp, "%s\n", a->arg);
 
-	fprintf(m->outputfp, "Content-Type: %s\n"
-			"Content-Transfer-Encoding: %s\n\n",
-			m->mimetype, m->mimeencoding);
+	fprintf(m->outputfp, "Content-Type: %s", m->mimetype);
+	if (m->textplaincharset && *m->textplaincharset)
+	{
+		const char *c;
+
+		fprintf(m->outputfp, "; charset=\"");
+		for (c=m->textplaincharset; *c; c++)
+		{
+			if (*c != '"' && *c != '\\')
+				putc(*c, m->outputfp);
+		}
+		fprintf(m->outputfp, "\"");
+	}
+
+	if (m->contentname && *m->contentname)
+	{
+		const char *c;
+
+		fprintf(m->outputfp, "; name=\"");
+		for (c=m->contentname; *c; c++)
+		{
+			if (*c != '"' && *c != '\\')
+				putc(*c, m->outputfp);
+		}
+		fprintf(m->outputfp, "\"");
+	}
+
+	fprintf(m->outputfp, "\nContent-Transfer-Encoding: %s\n\n",
+		m->mimeencoding);
 
 	if (strcmp(m->mimeencoding, "quoted-printable") == 0)
 	{
